module.exports = (node, graph) => {
  // create input trigger port we will use to receive data from parent node
  const triggerIn = node.triggerIn("in");
  // create output trigger port we will use to send data to child nodes
  const triggerOut = node.triggerOut("out");

  // define the in port for the mouse positions
  const mouseIn = node.in("mouse", null);

  // size of each rectangle
  const step = 50;
  // margin size in pixels
  const margin = 20;

  triggerIn.onTrigger = (props) => {
    const { canvas, ctx } = props;

    // save canvas state
    ctx.save();

    // start drawing clipping mask
    ctx.beginPath();

    // draw clipping mask rectangle slightly smaller than whole page
    ctx.rect(
      margin,
      margin,
      canvas.width - margin * 2,
      canvas.height - margin * 2
    );
    ctx.clip();

    // draw a grid moving by "step" pixels to the right and down in a loop
    for (let x = 0; x < canvas.width; x += step) {
      for (let y = 0; y < canvas.height; y += step) {
        // save current canvas state
        ctx.save();

        // move "drawing pen" to x, y positions
        ctx.translate(x, y);

        // set fill color to Nodes Blue
        ctx.fillStyle = "#0001f2";

        // draw rectangle at the current "pen position"
        ctx.fillRect(1, 1, step - 2, step - 2);

        // restore canvas state to before translation
        ctx.restore();
      }
    }

    const mouseState = mouseIn.value;

    let index = 0;
    for (var x = 0; x < canvas.width; x += step) {
      for (var y = 0; y < canvas.height; y += step) {
        ctx.save();
        ctx.translate(x, y);

        // move to the center of the grid cell
        ctx.translate(step / 2, step / 2);

        // rotate the shapes based on mouse y position
        ctx.rotate(mouseState.y / 100);

        // trigger output port effectively drawing child nodes
        // with additional `size` and `index` properties
        triggerOut.trigger({
          ...props,
          index,
          // change shape size based on mouse x position
          size: step * 0.2 + (mouseState.x / canvas.width) * 100,
        });
        ctx.restore();
        index++;
      }
    }

    ctx.restore();
  };
};
